#ifndef Client_HPP
#define Client_HPP

#include <boost/noncopyable.hpp>
#include <boost/thread.hpp>
//#include <boost/thread/mutex.hpp>
//#include <boost/thread/thread.hpp>

#include <msgClient.hh>

namespace hatptester{

//TODO Waits for connection if connection to server is lost

class HATPClient{
private:
	//Name of the HATPOnboard client
	#define HATPName "HATP"

	const std::string name;
	bool exit;
	boost::mutex mutexExit; //Is used when changing the value of exit
	
	msgClient client;
	boost::mutex mutexClient;
	boost::condition_variable conditionClient; //Is used to start the thread (says the client is ready)
	
	std::string message;
	bool status;
	boost::mutex mutexMessageReceiver;
	boost::mutex mutexMessageReady;
	boost::condition_variable conditionMessageReady;
	
	boost::thread thread;

	HATPClient();
	HATPClient(const HATPClient&);
	const HATPClient& operator=(const HATPClient&);
	
public:
	HATPClient(const std::string& name, const std::string& serverName, unsigned int port):
		name(name), exit(false), thread(boost::bind(&HATPClient::threadMessageReceiver, this)){
		//First wait for the thread to be ready
		{
			boost::mutex::scoped_lock lock(mutexClient);
			conditionClient.wait(lock);
		}
		
		//Then connect the client
		unsigned short attempts=0;
		bool clientConnected;
		
		{
			boost::mutex::scoped_lock lock(mutexClient);
			clientConnected=client.isConnected();
		}
		
		while(!clientConnected && attempts<5){
			boost::mutex::scoped_lock lock(mutexClient);
			client.connect(name, serverName, port);
			if(!client.isConnected()){
				usleep(200);
				++attempts;
			}
			clientConnected=client.isConnected();
		}
		conditionClient.notify_one();
		if(!client.isConnected()){
			stopThread();
		}
	}
	
	bool isConnected(){
		boost::mutex::scoped_lock lock(mutexClient);
		return (!exit && client.isConnected());
	}
	
	bool sendMessage(const std::string& message){
		boost::mutex::scoped_lock lock(mutexClient);
		if(client.isConnected()){
			return client.sendMessage(HATPName, message);
		}
		return false;
	}
	
	std::pair<bool, std::string> requestValue(const std::string& request){//Sends a message to HATP and wait for its answer
		if(sendMessage(request)){
			bool success;
			//Get the message from he thread
			{
				boost::mutex::scoped_lock lock(mutexMessageReady);
				success = conditionMessageReady.timed_wait(lock, boost::posix_time::milliseconds(500));
			}
			
			{
				if(success){
					boost::mutex::scoped_lock lock(mutexMessageReceiver);
					if(status){
						return std::make_pair(true, message);
					}
				}
			}
		}
		return std::make_pair(false, "");
		
	}
		
	void threadMessageReceiver(){
		conditionClient.notify_one(); //Thread is ready
		
		//Wait for connection
		{
			boost::mutex::scoped_lock lock(mutexClient);
			conditionClient.wait(lock);
		}
	
		std::pair<std::string, std::string> messageTemp;
		bool localExit=false;
		
		{
			boost::mutex::scoped_lock lock(mutexClient);
			if(!client.isConnected()){
				localExit=true;
			}
		}
		
		while(!localExit) {
			bool sleep=false;
			mutexExit.lock();
			localExit=exit;
			mutexExit.unlock();
			
			if(!localExit){
				boost::mutex::scoped_lock lock(mutexClient);
				if(client.isConnected()){
					if(client.isMessageWaiting()){
						messageTemp = client.getMessage();
						if(messageTemp.first!=""){
							boost::mutex::scoped_lock lock(mutexMessageReceiver);
							message=messageTemp.second;
							status=true;
							
							conditionMessageReady.notify_one();
						}else{
							boost::mutex::scoped_lock lock(mutexMessageReceiver);
							message="";
							status=false;
							
							conditionMessageReady.notify_one();
						}
						//Add all the cases : errors, quit, ...
					}else{
						sleep=true;
					}
				}else{
					localExit=true;
				}
			}
			
			if(sleep){
				usleep(200);
			}
		}
	}
	
	void stopThread(){
		mutexExit.lock();
		bool localExit=exit;
		mutexExit.unlock();
		if(localExit==false){
			mutexExit.lock();
			exit=true;
			mutexExit.unlock();
			thread.join();
		}
	}
	
	~HATPClient(){
		stopThread();
	}
};

} //namespace

#endif //Client_HPP
