#include <View.hpp>

namespace hatptester{

MainWindow::MainWindow():QMainWindow(), 
	controller(HATPController::getInstance()),
	centralPart(this),
	taskBox("Task selection"),
	taskView(&taskBox),
	defaultText("Parameters for the task : "),
	informationsText("Please select a task (a method or an Action)\n\n\n\n\n\n"),
	quitAction("&Quit", this), sendAction("&Send request", this),
	aboutAction("&About", this),
	sendButton("Send request", this){
	//First check if the controller is ready (its client is connected), if not leave
	if(!controller.isReady()){
		std::cerr<<"Failed to connect to the server or could not initialise. Are MsgServer and HATPOnbord running ?"<<std::endl;
		exit(99);
	}
	connect(&controller, SIGNAL(sendStop()), qApp, SLOT(quit()));

	//Actions
	quitAction.setShortcut(QKeySequence("Ctrl+C"));
	//TODO Change back to "Ctrl+Q"
	quitAction.setToolTip("Exit HATP Tester");
	quitAction.setIcon(QIcon::fromTheme("application-exit"));
	connect(&quitAction, SIGNAL(triggered()), qApp, SLOT(quit()));
	
	sendAction.setShortcut(QKeySequence("Ctrl+S"));
	sendAction.setToolTip("Send the current request to HATP");
	sendAction.setEnabled(false);
//TODO	sendAction.setIcon(QIcon::fromTheme(""));
	connect(&sendAction, SIGNAL(triggered()), this, SLOT(sendRequest()));
	
	connect(&aboutAction, SIGNAL(triggered()), this, SLOT(about()));
	
	//Menu
	QMenu *fileMenu = menuBar()->addMenu("&File");
	fileMenu->addAction(&sendAction);
	fileMenu->addSeparator();
	fileMenu->addAction(&quitAction);
	
	QMenu *helpMenu = menuBar()->addMenu("&Help");
	helpMenu->addAction(&aboutAction);
	
	//Top left part (Tasks' list)
	taskView.setModel(&controller.getTasks());
	taskView.setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOn);
	taskView.setEditTriggers(QAbstractItemView::NoEditTriggers);
	connect(&taskView, SIGNAL(clicked(const QModelIndex&)), this, SLOT(clicTask(const QModelIndex&)));
	
	QVBoxLayout* topLeftBox=new QVBoxLayout();
	topLeftBox->addWidget(&taskView);
	taskBox.setLayout(topLeftBox);
	
	//Top right part (Parameters for the selected task)
	QVBoxLayout* topRightBox=new QVBoxLayout();
	QStandardItemModel* model=new QStandardItemModel(5,2);
	QStringList labels;
	labels<<"Name"<<"Value";
	model->setHorizontalHeaderLabels(labels);
	QStringList typeLabels;
	typeLabels<<"Type"<<"Type"<<"Type"<<"Type"<<"Type";
	model->setVerticalHeaderLabels(typeLabels);
	parametersView.setModel(model);
	ParameterDelegate* delegate=new ParameterDelegate();
	connect(delegate, SIGNAL(requestIsReady()), this, SLOT(activateSend()));
	connect(delegate, SIGNAL(modelUpdated()), this, SLOT(updateInformations()));
	parametersView.setItemDelegate(delegate);
	parametersView.setSelectionMode(QAbstractItemView::SingleSelection);
	parametersView.horizontalHeader()->setResizeMode(QHeaderView::ResizeToContents);
	parametersView.horizontalHeader()->setStretchLastSection(true);
	topRightBox->addWidget(&parametersView);
	
	//TODO Use the metric for both to prevent some weird cases
	std::string longestName=defaultText+controller.getLongestTaskName();
	QFontMetrics metric(QFont("Times", 14, QFont::Bold));
	QRect rect(metric.boundingRect(longestName.c_str()));
	parameterBox.setMinimumWidth(rect.width());
	
	parameterBox.setTitle(defaultText.c_str());
	parameterBox.setLayout(topRightBox);
	
	//Bottom left part (Informations)
	QGroupBox* informationsBox=new QGroupBox("Informations");
	QVBoxLayout* bottomLeftLayout=new QVBoxLayout();
	informationsText.setWordWrap(true);
	bottomLeftLayout->addWidget(&informationsText);
	informationsBox->setLayout(bottomLeftLayout);
	
	//Bottom right part (Send button)
	sendButton.setToolTip("Send the current request to HATP");
	sendButton.setEnabled(false);
	sendButton.setMinimumSize(sendButton.sizeHint());
	connect(&sendButton, SIGNAL(clicked()), this, SLOT(sendRequest()));
	QGridLayout* bottomRightLayout=new QGridLayout();
	bottomRightLayout->addWidget(new QLabel(""), 0, 0); //To insert space
	bottomRightLayout->addWidget(new QLabel(""), 0, 1); //To insert second space
	bottomRightLayout->addWidget(&sendButton, 0, 2);
	
	QGridLayout* centralPartLayout=new QGridLayout;
	centralPartLayout->addWidget(&taskBox, 0, 0);
	centralPartLayout->addWidget(&parameterBox, 0, 1);
	centralPartLayout->addWidget(informationsBox, 1, 0);
	centralPartLayout->addLayout(bottomRightLayout, 1, 1);

	centralPart.setLayout(centralPartLayout);

	this->setMinimumSize(centralPart.sizeHint());
	setCentralWidget(&centralPart);
	
	show();
}

void MainWindow::resizeEvent(QResizeEvent* event){
	centralPart.resize(event->size());
}

//SLOTS
void MainWindow::about(){
	QMessageBox::about(this, "About hatptester", "<H1> hatptester </H1>"
		"The <b>hatptester</b> aims to provide a simple interface to "
		"send request to HATP Symbolic Planner.<br /><br />"
		"created by Raphaël Lallement < br />"
		"<i>raphael.lallement@laas.fr</i>"
	);
}

void MainWindow::sendRequest(){
	controller.sendRequest();
}

void MainWindow::activateSend(){
	sendAction.setEnabled(true);
	sendButton.setEnabled(true);
}

void MainWindow::updateInformations(){
	int jumps=5; //Represents the number of line to jump at the end to get a correct shape
	std::string informations;
	if(controller.isReadyCurrentTask()){
		informations="Request can be sent\n";
		
	}else{
		std::set<std::string> missingParameters=controller.getMissingParameters(controller.getCurrentTaskName());
		informations="To send the request, please set a value to the following parameters :\n";
		--jumps;
		for(std::set<std::string>::const_iterator it=missingParameters.begin(); it!=missingParameters.end(); ++it){
			informations+="\t"+*it+"\n";
			--jumps;
		}
	}
	while(jumps>0){
		informations+="\n";
		--jumps;
	}
	informationsText.setText(informations.c_str());
}

void MainWindow::clicTask(const QModelIndex& index)
{
	QVariant selectedTask(controller.getTasks().data(index, Qt::DisplayRole));
	std::string task(selectedTask.toString().toStdString());
	if(task=="" || task==" -- Methods : " || task==" -- Actions : "){
		return;
	}
	std::string title=defaultText+task;
	parameterBox.setTitle(title.c_str());
	
	QStandardItemModel* parameters=controller.getTaskParameter(task);
	if(parameters==NULL){
		parameters=new QStandardItemModel(5,2);
		sendAction.setEnabled(false);
		sendButton.setEnabled(false);
		informationsText.setText("Invalid task !");
	}else{
		sendAction.setEnabled(controller.isReadyTask(task));
		sendButton.setEnabled(controller.isReadyTask(task));
		updateInformations();
	}
	parametersView.setModel(parameters);
	parametersView.horizontalHeader()->setStretchLastSection(true);
}

};

int main(int argc, char *argv[])
{
	QApplication app(argc, argv);
 
	hatptester::MainWindow window;
	
	return app.exec();
}
